@@ -20,12 +20,15 @@ pub enum Status {
     Error(String),
 }
 
+/// Defines a stage to be run by runner.
 #[derive(Debug)]
 pub struct StageSpec {
     pub(crate) image: String,
     pub(crate) extend: Vec<String>,
+    pub(crate) workdir: String,
     pub(crate) script: Vec<String>,
     pub(crate) envs: HashMap<String, String>,
+    pub(crate) mounts: HashMap<String, String>,
 }
 
 /// Task runner that prepares for the task, runs the task, tracks running state,
@@ -77,14 +80,13 @@ impl<T: Reporter> Runner<'_, T> {
         log::info!("run stage `{}` with image: {}", name, image);
         self.set_status(Status::RunStage(name.into()))?;
 
-        let dir = tempfile::tempdir().map_err(Error::IOError).handle(self)?;
-
         self.sandbox
             .run(
                 &image,
+                &stage.workdir,
                 &stage.script,
                 &stage.envs,
-                dir.path(),
+                &stage.mounts,
                 &self.reporter,
             )
             .await
@@ -113,19 +115,30 @@ impl<T: Reporter> Drop for Runner<'_, T> {
 pub struct HandledError(pub Error);
 
 trait ErrorHandler<T> {
-    fn handle<TR: Reporter>(self, runner: &mut Runner<TR>) -> Result<T, HandledError>;
+    fn handle(self, runner: &mut Runner<impl Reporter>) -> Result<T, HandledError>;
     fn ignore(self) -> Result<T, HandledError>;
 }
 
-impl<T> ErrorHandler<T> for Result<T, Error> {
-    fn handle<TR: Reporter>(self, r: &mut Runner<TR>) -> Result<T, HandledError> {
-        if let Err(e) = &self {
-            r.set_status(Status::Error(format!("{:?}", e)))?;
+impl<T, E> ErrorHandler<T> for Result<T, E>
+where
+    E: Into<Error> + std::fmt::Debug,
+{
+    fn handle(self, r: &mut Runner<impl Reporter>) -> Result<T, HandledError> {
+        match self {
+            Err(e) => {
+                r.set_status(Status::Error(format!("{:?}", e)))?;
+                Err(HandledError(e.into()))
+            }
+            Ok(r) => Ok(r),
         }
-        // now error has been reported
-        self.map_err(HandledError)
     }
     fn ignore(self) -> Result<T, HandledError> {
-        self.map_err(HandledError)
+        self.map_err(|e| HandledError(e.into()))
+    }
+}
+
+impl From<HandledError> for Error {
+    fn from(e: HandledError) -> Self {
+        e.0
     }
 }
