@@ -26,17 +26,16 @@ impl Sandbox<'_> {
 
     /// Build docker image and return image ID.
     pub async fn build(&self, image: &str, extend: &[String]) -> Result<String, Error> {
-        let dir = tempfile::tempdir().map_err(Error::IOError)?;
+        let dir = tempfile::tempdir()?;
         let dir_path = dir.path().to_str().expect("tempdir should always be valid");
 
         {
             let file_path = dir.path().join("Dockerfile");
             log::debug!("writing Dockerfile at: {:?}", file_path);
-            let mut file = File::create(file_path).map_err(Error::IOError)?;
-            writeln!(file, "FROM {}", image).map_err(Error::IOError)?;
+            let mut file = File::create(file_path)?;
+            writeln!(file, "FROM {}", image)?;
             if !extend.is_empty() {
-                writeln!(file, "RUN {}", extend.join(" && ").replace('\n', ""))
-                    .map_err(Error::IOError)?;
+                writeln!(file, "RUN {}", extend.join(" && ").replace('\n', ""))?;
             }
         }
 
@@ -83,40 +82,60 @@ impl Sandbox<'_> {
     }
 
     /// Run scripts with envs.
-    pub async fn run<T: Reporter>(
+    pub async fn run(
         &self,
         image: &str,
+        workdir: &str,
         script: &[String],
         envs: &HashMap<String, String>,
-        dir_path: &Path,
-        reporter: &T,
+        mounts: &HashMap<String, String>,
+        reporter: &impl Reporter,
     ) -> Result<(), Error> {
         log::info!("create container using {} with envs {:?}", image, envs);
 
+        let temp = tempfile::tempdir()?;
+        let asset_path = temp.path();
+
         {
-            let file_path = dir_path.join(".run.sh");
+            let file_path = asset_path.join(".run.sh");
             log::debug!("writing stage script at: {:?}", file_path);
-            let mut file = File::create(file_path).map_err(Error::IOError)?;
+            let mut file = File::create(file_path)?;
             for line in script {
-                writeln!(file, "{}", line).map_err(Error::IOError)?;
+                writeln!(file, "{}", line)?;
             }
-            file.flush().map_err(Error::IOError)?;
+            file.flush()?;
+        }
+
+        let mut binds: Vec<String> = vec![];
+        binds.push(format!(
+            "{}:/assets",
+            asset_path
+                .to_str()
+                .expect("path should always be valid utf-8 string")
+        ));
+        for (k, v) in mounts {
+            // TODO: check for permission
+            binds.push(format!(
+                "{}:{}:ro",
+                Path::new(v)
+                    .canonicalize()?
+                    .to_str()
+                    .expect("path should always be valid utf-8 string"),
+                k,
+            ));
         }
 
         let options = ContainerOptions::builder(image)
-            .volumes(vec![&format!(
-                "{}:/app",
-                dir_path.to_str().expect("path should always be valid")
-            )])
-            .working_dir("/app")
-            .cmd(vec!["sh", "-e", ".run.sh"])
+            .volumes(binds.iter().map(|s| s as &str).collect())
+            .working_dir(workdir)
+            .cmd(vec!["sh", "-e", "/assets/.run.sh"])
             .env(
                 // TODO: probably better solution needed
                 envs.iter()
                     .map(|(k, v)| format!("{}={}", k, v))
                     .collect::<Vec<String>>()
                     .iter()
-                    .map(AsRef::as_ref)
+                    .map(|s| s as &str)
                     .collect::<Vec<&str>>(),
             )
             .attach_stdout(true)
@@ -129,17 +148,12 @@ impl Sandbox<'_> {
             .auto_remove(true)
             .build();
 
-        let container = self
-            .docker
-            .containers()
-            .create(&options)
-            .await
-            .map_err(Error::DockerError)?;
+        let container = self.docker.containers().create(&options).await?;
 
         log::info!("created container with id: {}", container.id);
 
         let container = self.docker.containers().get(&container.id);
-        container.start().await.map_err(Error::DockerError)?;
+        container.start().await?;
 
         log::info!("container started");
 
@@ -149,7 +163,7 @@ impl Sandbox<'_> {
         let (log, exit) = join(log_op, wait_op).await;
 
         let _ = log?;
-        let e = exit.map_err(Error::DockerError)?;
+        let e = exit?;
 
         log::info!("container exited with code {}", e.status_code);
         if e.status_code > 0 {
@@ -164,10 +178,10 @@ impl Sandbox<'_> {
         Ok(())
     }
 
-    async fn process_logs<T: Reporter>(
+    async fn process_logs(
         &self,
         container: &Container<'_>,
-        reporter: &T,
+        reporter: &impl Reporter,
     ) -> Result<(), Error> {
         let mut stream = container.logs(
             &LogsOptions::builder()
@@ -184,15 +198,15 @@ impl Sandbox<'_> {
             if limit < 0 {
                 break;
             }
-            let chunk = exec_result.map_err(Error::DockerError)?;
+            let chunk = exec_result?;
             match chunk {
                 TtyChunk::StdOut(bytes) => {
-                    let line = from_utf8(&bytes).map_err(Error::EncodingError)?;
+                    let line = from_utf8(&bytes)?;
                     log::debug!("stdout | {}", line.trim_end());
                     reporter.emit_stdout(line)?
                 }
                 TtyChunk::StdErr(bytes) => {
-                    let line = from_utf8(&bytes).map_err(Error::EncodingError)?;
+                    let line = from_utf8(&bytes)?;
                     log::debug!("stderr | {}", line.trim_end());
                     reporter.emit_stderr(line)?
                 }
